-------------------------------------------------------------------------------
-- Title      : AW Router
-- Project    : AXI-3 Crossbar Switch
-------------------------------------------------------------------------------
-- File	      : master2slave/axi3-interconnect-aw-router.vhd
-- Author     : Mario Hüttel <mario.huettel@gmx.net>
-- Standard   : VHDL'93/02
-------------------------------------------------------------------------------
-- Description: 
-------------------------------------------------------------------------------
-- Copyright (c) 2016
--
-- This file is part of AXI-3 Crossbar Switch.
--
-- AXI-3 Crossabr Switch is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation,  version 2 of the License.
--
-- This code is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this code.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.axi3intercon_pkg.all;
use work.axi_aw_router_pkg.all;

entity axi3intercon_aw_router is
	port(
		aclk	       : in  std_logic;
		rst	       : in  std_logic;
		masters_out    : in  axi_aw_masters_out_t(0 to MASTER_COUNT - 1);
		masters_in     : out axi_aw_masters_in_t(0 to MASTER_COUNT - 1);
		slaves_out     : in  axi_aw_slaves_out_t(0 to SLAVE_COUNT);
		slaves_in      : out axi_aw_slaves_in_t(0 to SLAVE_COUNT);
		write_locks    : out write_locks_t(0 to MASTER_COUNT - 1);  -- write_* signals come/go from/to the write router.
		write_releases : in  write_releases_t;
		address_array  : in  axi_slave_addresses_t(0 to SLAVE_COUNT - 1);
		mask_array     : in  axi_slave_addresses_t(0 to SLAVE_COUNT - 1)
		);
end entity axi3intercon_aw_router;

architecture RTL of axi3intercon_aw_router is
	alias clk is aclk;
	type aw_state_t is (AW_READY, AW_ACTIVE, AW_BLOCK);
	type aw_states_t is array (0 to MASTER_COUNT - 1) of aw_state_t;
	signal aw_states     : aw_states_t;
	signal write_locks_s : write_locks_t(0 to MASTER_COUNT - 1);
	signal awready_s     : std_logic_vector(0 to MASTER_COUNT - 1);
begin
	write_locks <= write_locks_s;

	awready_gen : for i in 0 to MASTER_COUNT - 1 generate
		masters_in(i).awready <= awready_s(i);
	end generate awready_gen;

	aw_router : process(clk, rst) is
		variable slave_idx    : integer range 0 to SLAVE_COUNT := SLAVE_COUNT;	-- 1 more slave than connected for handling bad requests.
		variable slave_in_use : std_logic_vector(0 to SLAVE_COUNT);

	begin
		if rst = '1' then
			for i in 0 to MASTER_COUNT - 1 loop
				aw_states(i)		   <= AW_READY;
				write_locks_s(i).slave_idx <= 0;
				write_locks_s(i).locked	   <= '0';
			end loop;
			for i in 0 to SLAVE_COUNT loop
				slaves_in(i).awvalid <= '0';
				slaves_in(i).awaddr  <= (others => '0');
				slaves_in(i).awburst <= (others => '0');
				slaves_in(i).awcache <= (others => '0');
				slaves_in(i).awid    <= (others => '0');
				slaves_in(i).awlen   <= (others => '0');
				slaves_in(i).awlock  <= (others => '0');
				slaves_in(i).awprot  <= (others => '0');
				slaves_in(i).awsize  <= (others => '0');
			end loop;
			awready_s    <= (others => '0');
			slave_in_use := (others => '0');
		elsif rising_edge(clk) then
			for i in 0 to MASTER_COUNT - 1 loop  -- Loop for every master
				case aw_states(i) is
					when AW_READY =>
						if masters_out(i).awvalid = '1' then  -- check awready. just to prevent glitches
							slave_idx := calculate_slave(masters_out(i).awaddr, address_array, mask_array);
							if slave_in_use(slave_idx) /= '1' then
								awready_s(i)		   <= '1';
								write_locks_s(i).slave_idx <= slave_idx;
								write_locks_s(i).locked	   <= '1';
								slave_in_use(slave_idx)	   := '1';
					-- output request to slave
								if slave_idx = SLAVE_COUNT then
									slaves_in(slave_idx).awaddr <= masters_out(i).awaddr;
								else
									slaves_in(slave_idx).awaddr <= masters_out(i).awaddr and (not mask_array(slave_idx));
								end if;
								slaves_in(slave_idx).awid    <= std_logic_vector(to_unsigned(i, WID_SLAVE_BITS - WID_MASTER_BITS)) & masters_out(i).awid;
								slaves_in(slave_idx).awburst <= masters_out(i).awburst;
								slaves_in(slave_idx).awcache <= masters_out(i).awcache;
								slaves_in(slave_idx).awlen   <= masters_out(i).awlen;
								slaves_in(slave_idx).awlock  <= masters_out(i).awlock;
								slaves_in(slave_idx).awprot  <= masters_out(i).awprot;
								slaves_in(slave_idx).awsize  <= masters_out(i).awsize;
								slaves_in(slave_idx).awvalid <= '1';
							     -- Active state waits for awready to be asserted
								aw_states(i)		     <= AW_ACTIVE;
							end if;
						end if;
					when AW_ACTIVE =>
						awready_s(i) <= '0';
						if slaves_out(write_locks_s(i).slave_idx).awready = '1' then
							slaves_in(write_locks_s(i).slave_idx).awvalid <= '0';
							aw_states(i)				      <= AW_BLOCK;
						end if;
					when AW_BLOCK =>
						if write_releases(i) = '1' then
							slave_in_use(write_locks_s(i).slave_idx) := '0';
							write_locks_s(i).locked			 <= '0';
							aw_states(i)				 <= AW_READY;
						end if;
				end case;

			end loop;
		end if;
	end process aw_router;

end architecture RTL;
