-------------------------------------------------------------------------------
-- Title      : B Router
-- Project    : AXI-3 Crossbar Switch
-------------------------------------------------------------------------------
-- File       : slave2master/axi3-interconnect-b-router.vhd
-- Author     : Mario Hüttel <mario.huettel@gmx.net>
-- Standard   : VHDL'93/02
-------------------------------------------------------------------------------
-- Description: 
-------------------------------------------------------------------------------
-- Copyright (c) 2016
--
-- This file is part of AXI-3 Crossbar Switch.
--
-- AXI-3 Crossabr Switch is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation,  version 2 of the License.
--
-- This code is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this code.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.axi3intercon_pkg.all;
use work.axi_b_router_pkg.all;

entity axi3_intercon_b_router is
	port(
		clk         : in  std_logic;
		rst         : in  std_logic;
		slaves_in   : out axi_b_slaves_in_t(0 to SLAVE_COUNT);
		slaves_out  : in  axi_b_slaves_out_t(0 to SLAVE_COUNT);
		masters_in  : out axi_b_masters_in_t(0 to MASTER_COUNT - 1);
		masters_out : in  axi_b_masters_out_t(0 to MASTER_COUNT - 1)
	);
end entity axi3_intercon_b_router;

architecture RTL of axi3_intercon_b_router is
	type master_indexes_t is array (natural range <>) of integer range 0 to MASTER_COUNT - 1;
	type b_state_t is (B_READY, B_ACTIVE);
	type b_states_t is array (0 to SLAVE_COUNT) of b_state_t;
	signal b_states : b_states_t;
begin
	b_router : process(clk, rst) is
		variable master_in_use : std_logic_vector(0 to MASTER_COUNT - 1) := (others => '0');
		variable master_idx    : master_indexes_t(0 to SLAVE_COUNT);
	begin
		if rst = '1' then
			master_in_use := (others => '0');
			for i in 0 to SLAVE_COUNT loop
				slaves_in(i).bready <= '0';
				b_states(i)         <= B_READY;
			end loop;
			for i in 0 to MASTER_COUNT - 1 loop
				masters_in(i).bvalid <= '0';
				masters_in(i).bid    <= (others => '0');
				masters_in(i).bresp  <= (others => '0');
				masters_in(i).buser  <= (others => '0');
			end loop;
		-- TODO: Reset
		elsif rising_edge(clk) then
			for i in 0 to SLAVE_COUNT loop
				case b_states(i) is
					when B_READY =>
						if slaves_out(i).bvalid = '1' then
							-- calculate master	
							master_idx(i) := to_integer(unsigned(slaves_out(i).bid(RID_SLAVE_BITS - 1 downto RID_MASTER_BITS)));
							if (master_in_use(master_idx(i)) = '0') then
								master_in_use(master_idx(i))     := '1';
								slaves_in(i).bready              <= '1';
								masters_in(master_idx(i)).bid    <= slaves_out(i).bid(masters_in(master_idx(i)).bid'range);
								masters_in(master_idx(i)).bresp  <= slaves_out(i).bresp;
								masters_in(master_idx(i)).buser  <= slaves_out(i).buser;
								masters_in(master_idx(i)).bvalid <= '1';
								b_states(i)                      <= B_ACTIVE;
							end if;
						end if;
					when B_ACTIVE =>
						slaves_in(i).bready <= '0';
						if masters_out(master_idx(i)).bready = '1' then
							masters_in(master_idx(i)).bvalid <= '0';
							master_in_use(master_idx(i))     := '0';
							b_states(i)                      <= B_READY;
						end if;
				end case;
			end loop;
		end if;
	end process b_router;

end architecture RTL;
